// This is readln.cpp, the primary input transmitter.
//
// Joachim Wuttke aug01 Provides readline and history support.
// sept 01 support for escape commands.

#include "mystd.h"
#include <iostream>
#include <map>
#include <readline/readline.h>
#include <readline/history.h>


typedef string (mac_fct)(string);

class CMacro {
public:
    inline CMacro(string t_in, mac_fct *a_in) {
        text = t_in; 
        action = a_in; 
    };
    string   text;
    mac_fct  *action;
};

typedef map<string, CMacro> CMMap;
typedef map<string, CMacro>::iterator IMMap;

namespace NRead {

    // user interface, made available through readln.h:
    string readln(const string&);
    string readwd(const string&);
    string pop(); // return stack up to eol
    void clear(void);
    string exemac(const string);
    int dofmac(const string in, double *out);
    int wofmac(const string in, string *out);
    bool nonempty( void );

    // internals, used only in this file:
    bool initialized=0;
    bool video=1; 
    string stack;
    string halfline;
    CMMap mmap;

    void initialize(void);
    string readline_interface(const string);
    void register_macros(void);
    string scan_macro(const string);
    // mac_fct *mac_incl;

    // the following are only called indirectly through macros:
    string mac_ask(string r);
    string mac_def(string r);
    string mac_incl(string r);
    string mac_list(string r);
    string mac_video(string r);
    string mac_wait(string r);
};


string NRead::readln(const string& prompt)
{
    if (!initialized) initialize();

    if (halfline==string("")) {
        bool interactive = !stack.size();
        if (interactive) stack = readline_interface(prompt);
        stack = scan_macro(stack);
        // cout << "DEBUG readln: after scan_mac [" << stack << "]\n";
        mystd::string_extract_line(stack, &halfline, &stack);
        if (!interactive && video) 
            cout << prompt << " < " << halfline << "\n";
    } 

    if (halfline[0]=='`') {
        return readwd(prompt);
    }

    string ret = halfline;
    halfline = string("");
    return ret;
}

string NRead::readwd(const string& prompt)
{
    string h, ret;
    if (halfline==string("")) {
        halfline = readln(prompt);
    }

    mystd::string_extract_word(halfline, &ret, &h);
    halfline = h;
    return ret;
}

string NRead::pop()
{
    string ret = halfline;
    halfline = "";
    return ret;
}

void NRead::clear(void)
{
    stack = "";
    halfline = "";
}

string NRead::exemac(const string in)
{
    if (!initialized) initialize();
    return scan_macro(in); // aber ob das so einfach bleibt ?
}

int NRead::dofmac(const string in, double *out)
{
    string s = exemac(in);
    if (sscanf(s.c_str(), "%lf", out)!=1) return -1;
    return 0;
}

int NRead::wofmac(const string in, string *out)
{
    string s = exemac(in);
    *out = s;
    if (s==string("")) return -1;
    return 0;
}

bool NRead::nonempty( void )
{
    return halfline!="";
}

void NRead::initialize(void)
{
    using_history();   // initialization for libhistory
    register_macros();
}

string NRead::readline_interface(const string prompt)
{
    char *rr;
    char p[60]; // needed for DEBIAN 2.2r3 version of readline
    string r;
    strncpy(p, prompt.c_str(), 60);
    p[60] = 0;
    // printf("%s\n",p); scanf("%s", p); return p; // ohne readline
    rr = readline(p); // from libreadline
    add_history(rr);               // from libhistory
    r = string(rr);
    free(rr);
    return r;
}

string NRead::scan_macro(const string r)
{
    string::size_type i=0,j;
    IMMap pos;
    string r0, r1, m;

    // cout << "DEBUG scan_macro ["<<r<<"]\n";
    while(1) {
        if      (i>=r.size() || r[i]=='\n')
            return r; 
        else if (r[i]=='\\')
            break;
        i++;
    } 
    // cout << "DEBUG: macro starts at [" << r.substr(i) << "]\n";
    r0 = r.substr(0,i);

    if        (i+1==r.size()) {
        return r0 + "\\EOL"; // needed by sask
    } else if (r[i+1]=='\n') {
        return r0 + "\\EOL" + scan_macro(r.substr(i+1));
    } else if (r[i+1]==' ') {
        return r0 + " " + scan_macro(r.substr(i+2));
    } else if (r[i+1]=='\\') {
        return r0 + "\\" + scan_macro(r.substr(i+2));
    }
    // so far the special cases.

    j = r.find_first_of(" \n\t\\{}[]()=+-", i+1);
    if (j==string::npos) j = r.size();
    m = r.substr(i+1, j-i-1);
    // cout << "DEBUG: found macro [" << m << "]\n";
    while (j< r.size() && r[j]==' ') j++;

    // r1 = (j<r.size()) ? r.substr(j) : string("");
    r1 = r.substr(j);
    // cout << "DEBUG: fragments ["<<r0<<"]+macro+["<<r1<<"]\n";

    if ((pos = mmap.find(m)) == (mmap.end())) {
        cout << "! macro \\" << m << " undefined\n";
        return string ("");
    }
    if (*(pos->second.action))
        return r0 + scan_macro((*(pos->second.action))(scan_macro(r1)));
    else
        return r0 + scan_macro(pos->second.text + r1);
}

string NRead::mac_ask(string r)
{
    string a;
    mystd::string_extract_word(r, &a, &r);

    string s = readline_interface(string("* -> macro input request \"") +
                                  a + string("\": ")); 
    return s + r;
}

string NRead::mac_def(string r)
{
    string name, value;
    mystd::string_extract_word(r, &name, &r);
    mystd::string_extract_line(r, &value, &r);

    // cout << "going to define ["<<name<<"] as ["<<value<<"]\n";
    mmap.insert(make_pair(name, CMacro(value, 0)));

    return r;
}

string NRead::mac_incl(string r)
{
    string fshort, flong, s;
    ifstream *fd;
    mystd::string_extract_word(r, &fshort, &r);
    // cout << "going to include ["<<a<<"]\n";

    if (mystd::glob_one_file(fshort, "", "", &flong)!=1 &&
        mystd::glob_one_file(fshort, "", "mac", &flong)!=1 &&
        mystd::glob_one_file(fshort, "~/mac", "", &flong)!=1 &&
        mystd::glob_one_file(fshort, "~/mac", "mac", &flong)!=1) {
        cout << "!! cannot locate include file [" << fshort << "]\n";
        goto myend;
    }

    if (!(fd = new ifstream(flong.c_str()))) {
        cout << "!! cannot open include file [" << flong << "]\n";
        goto myend;
    }
    char c;
    while (fd->get(c)) s += c;
    delete fd;
    // cout << "read [" << s << "]\n";
 myend:
    return s + r;
}

string NRead::mac_list(string r)
{
    IMMap pos;
    for (pos=mmap.begin(); pos!=mmap.end(); ++pos) {
        if (*(pos->second.action)) 
            printf("   %-12s %s\n", pos->first.c_str(), 
                   "<build-in function>");
        else
            printf("   %-12s %s\n", pos->first.c_str(), 
                   pos->second.text.c_str());
    }
    return r;
}

string NRead::mac_video(string r)
{
    string a;
    mystd::string_extract_word(r, &a, &r);

    if (a==string("0") || a==string("off"))
        video = 0;
    else if (a==string("1") || a==string("on"))
        video = 1;
    else
        cout << "!! usage: \\v 0 | 1 | off | on\n";
    return r;
}

string NRead::mac_wait(string r)
{
    string a;
    mystd::string_extract_word(r, &a, &r);

    double sec;
    if (sscanf(a.c_str(), "%lg", &sec)!=1) {
        cout << "!! usage: \\w <sec>\n";
    } else {
        mystd::ssleep(sec);
    }
    return r;
}


void NRead::register_macros(void)
{
    mmap.insert(make_pair(string("a"),   CMacro("", mac_ask)));
    mmap.insert(make_pair(string("d"),   CMacro("", mac_def)));
    mmap.insert(make_pair(string("i"),   CMacro("", mac_incl)));
    mmap.insert(make_pair(string("l"),   CMacro("", mac_list)));
    mmap.insert(make_pair(string("v"),   CMacro("", mac_video)));
    mmap.insert(make_pair(string("w"),   CMacro("", mac_wait)));
    mmap.insert(make_pair(string("y"),   CMacro("TEST", 0)));
}
