#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <glob.h>
#include <iostream>
#include <boost/regex.hpp>
#include <boost/regex/v4/regex_match.hpp>

#include "myexception.h"
#include "mystd.h"

namespace mystd_aux {
    int tim2dbl(const string inp, double *val);
};

string strg(const string& val)
{
    // printf("DEBUG strg(%s)\n", val.c_str());
    return val;
}

string strg(const bool& val) {
    if (val)
        return "y";
    else
        return "n";
}

string strg(const double& val)
{
    // printf("DEBUG strg(%g)\n", val);
    char s[40];
    sprintf(s, "%g", val);
    return s;
}

string strg(const int& val)
{
    // printf("DEBUG strg(%d)\n", val);
    char s[40];
    sprintf(s, "%d", val);
    return s;
}

string strg(const uint& val)
{
    // printf("DEBUG strg(%u)\n", val);
    char s[40];
    sprintf(s, "%u", val);
    return s;
}

string strg(const char& val)
{
    // printf("DEBUG strg(%c)\n", val);
    return string(1, val);
}

double mystd::justnow(void)
{
    struct timeval exactlynow;
    gettimeofday(&exactlynow, NULL);
    return (double) exactlynow.tv_sec;
}

string mystd::myasctime(double time, int offs)
{
    long timin = (long) time;
    char buf[40];
    sprintf(buf, "%.24s", asctime(localtime(&timin)));
    return string(buf+offs);
}

void mystd::bell(void)
{
    static const char BEL = 0x7;
    cout << BEL;
}

#ifdef HAS_TIME
#include <time.h>     // nanosleep
void mystd::ssleep(double sec)
{
    struct timespec req;
    double fullsec;
    req.tv_nsec = (long) (1e9 * modf(sec, &fullsec));
    req.tv_sec = (time_t) fullsec;
    nanosleep(&req, NULL);
}
#endif

double mystd::dnint(double val)
{
    return floor(val+0.5);
}

int mystd::nint(double val)
{
    return (int) dnint(val);
}

const double mystd::pi = 3.1415926535897931;

double mystd::round(double val, double digits)
{
    if (val==0 || digits<=0) return 0;

    double v = fabs(val);
    int n = (int) floor ( digits - log10(v));
    // cout << " n=" << n;
    double p = pow(10., n);
    double r = dnint(v*p) / p;
    return (val<0 ? -1 : +1) * r;
}

int mystd::str2vec(string inp, vector<double> *V, uint nmax, bool force)
{
    using namespace mystd_aux;
    using namespace std;

    // cout << "DEBUG mystd [" << inp << "]\n";
    V->clear();

    while(inp.size() && isspace(inp[0])) inp.erase(inp.begin());
    while(inp.size() && strchr(" \t.\n",inp[inp.size()-1])) 
        inp.erase(inp.end()-1);

    // cout << "DEBUG  core [" << inp << "]\n";
    if (inp==string("")) {
        // cout << "DEBUG  empty\n";
        return 0;
    }
    // cout << "DEBUG  nonempty\n";

    string::size_type ji=0, jf;
    double val;
    char dummy;
    while (ji<inp.size()) {
        if ((jf=inp.find_first_of(", \t", ji)) == string::npos) 
            jf = inp.size();
        // cout << " look until " << jf << "\n";
        if (sscanf(inp.substr(ji,jf-ji).c_str(), "%lg%c", &val, &dummy)
            ==1) {
            V->push_back(val);
        } else if ( force ) {
            cout << "! str2vec: invalid value ["
                 << inp.substr(ji,jf-ji) << "] at pos " << ji
                 << " in [" << inp << "]\n";
            return -1;
        }
        if (nmax && V->size()>=nmax) break; // do not decode the tail
        // cout << " found "<<ji<<":"<<jf<<" value "<<val<<"\n";
        ji = inp.find_first_not_of(", \t", jf);
        // cout << " continue at " << ji << "\n";
    }
		
    return 0;
}

void mystd::merge(vector<double> *V, double val)
    // insert val into vector *V, presuming the latter is sorted
{
    for (uint i=0; i<V->size(); ++i) {
        if (val>(*V)[i]) continue;
        if (val==(*V)[i]) return;
        V->insert(V->begin()+i, val);
        return;
    }
    V->push_back(val);
}

int mystd::get_const(vector<double> *V, double *val)
    // check whether *V has a constant value; write this value to *val
{
    double v;
    if (V->size()==0) return -1; // empty vector
    v = (*V)[0];
    for (uint i=1; i<V->size(); ++i)
        if ((*V)[i]!=v) return 1; // vector not constant
    if (val) *val = v;
    return 0;
}

void mystd::unique(vector<double> *V, double tolabs, double tolrel)
{
    // tolrel z.Zt. nicht benutzt
    vector<double> aux;
    double diff;
    if (!V->size()) return;
    aux.push_back((*V)[0]);
    for (uint ii=1; ii<V->size(); ++ii)
        if((diff=fabs((*V)[ii]-aux.back()))>tolabs)
            aux.push_back((*V)[ii]);
    *V = aux;
}

double mystd::any2dbl( const string inp )
{
    if(!inp.size())
        throw BadData( "any2dbl expecting double, found empty input" );

    if(!strchr("+-0123456789.", inp[0]))
        throw BadData( "any2dbl expecting double, found bad char[0] in [" + inp + "]" );

    // first, catch private notation: "7m3" means 7*10^(-3)
    string::size_type j;
    double mant; int expt;
    if (sscanf(inp.c_str(), "%lgm%d", &mant, &expt)==2)
        return mant * pow(10.e0, (double)(-expt));

    // catch time and date:
//  if ((j=inp.find_first_of("dhm"))!=string::npos) { 
//        // possibly a date
//        return mystd_aux::tim2dbl( inp );

    // otherwise, expect regular real number:
    double val;
    if (sscanf(inp.c_str(), "%lg", &val)==1)
        return val;

    throw BadData ( "any2dbl expecting double, sscanf failed" );
}

int mystd::any2int( const string inp )
{
    if(!inp.size())
        throw BadData( "any2int expecting integer, found empty input" );

    if( boost::regex_match( inp, boost::regex("^\\s*[\\+\\-]?[0-9]+\\s*$") ) ){
        int val;
        if (sscanf(inp.c_str(), "%d", &val)==1)
            return val;
    }

    throw BadData( "any2int expecting integer, found [" + inp + "]" );
}

int mystd_aux::tim2dbl(const string inp, double *val)
{
    double del;
    int idel;
    *val = 0;
    bool isint = 1;
    string::size_type j1, jc;
    int unit=0;
    char c;

    for (j1=0, jc=0; jc<inp.size(); jc++) {
        if (strchr("+-0123456789", inp[jc])) continue;
        if (strchr(".eE", inp[jc])) {
            isint = 0;
            continue;
        }
        if (unit<=0 && inp[jc]=='d') {
            if (sscanf(inp.substr(j1, jc-j1).c_str(), "%d", &idel)
                ==1) {
                *val += 86400*idel;
            } else {
                cout << "failed to read #days in ["<<inp<<"]\n";
                return -11;
            }
            unit = 1;
            j1 = jc+1;
        } else if (unit<=1 && inp[jc]=='h') {
            if (sscanf(inp.substr(j1, jc-j1).c_str(), "%d", &idel)
                ==1) {
                *val += 3600*idel;
            } else {
                cout<<"failed to read #hours in ["<<inp<<"]\n";
                return -12;
            }
            unit = 2;
            j1 = jc+1;
        } else if (unit<=2 && inp[jc]=='m') {
            if (sscanf(inp.substr(j1, jc-j1).c_str(), "%d", &idel)
                ==1) {
                *val += 60*idel;
            } else {
                cout<<"failed to read #mins in ["<<inp<<"]\n";
                return -13;
            }
            unit = 2;
            j1 = jc+1;
        }
    }
    if (j1>=inp.size()) return 0;
    if (sscanf(inp.substr(j1).c_str(), "%lg%c", &del, &c)==1) {
        *val += del;
        return 0;
    } else {
        cout << "in time found invalid double in ["<<inp<<"]\n";
        return -14;
    }
}

void mystd::string_extract_word(const string in, string *out1, string *out2)
{
    // one word (*out1) is extracted; leading " \t" are retained.
    // in the tail (*out2) leading " \t" are deleted.
    // "\n" starts always a new word.
    // leading "`" makes *out1 go to next "`" or to end of line.

    string::size_type j0=0, j1, j2, jf=in.size();

    if (in[0]=='`') { // special case
        j1 = in.find_first_of('`', 1);
        if (j1==string::npos) {
            *out1 = in.substr(1);
            *out2 = "";
        } else {
            *out1 = in.substr(1,j1-1);
            j2 = in.find_first_not_of(" \t", j1+1);
            if (j2!=string::npos) 
                *out2 = in.substr(j2);
            else
                *out2 = "";
        }
        return;
    }		

    while (j0<jf && strchr(" \t",in[j0])) j0++;
    j1=j0+1;
    while (j1<jf && !strchr(" \t\n",in[j1])) j1++;
    j2=j1;
    while (j2<jf && strchr(" \t",in[j2])) j2++;
    *out1 = in.substr(0,j1);
    if (j2<jf) 
        *out2 = in.substr(j2);
    else
        *out2 = string("");
    //cout << "from ["<<in<<"] word ["<<*out1<<"] and tail ["<<*out2<<"]\n";
}

void mystd::string_extract_line(const string in, string *out1, string *out2)
{
    string line;
    string::size_type i = in.find('\n');
    if (i==string::npos) {
        *out1 = in;
        *out2 = string("");
    } else {
        *out1 = in.substr(0,i);
        *out2 = in.substr(i+1);
    }
    //cout << "from ["<<in<<"] line ["<<*out1<<"] and tail ["<<*out2<<"]\n";
}

void mystd::fname_divide( const string fname,
                          string *fdir, string *fshort, string *fext )
// divide "dir/short.ext" into "dir", "short", "ext"
// if some of the three return values are not needed, the user may supply 0
// instead of a pointer to a string.
{
    uint idir = fname.rfind("/") + 1;
    if( fdir ) *fdir = fname.substr( 0, idir );

    uint iext = fname.rfind(string(".")) + 1;
    if( iext<=idir ){
        if( fext ) *fext = "";
        if( fshort ) *fshort = fname.substr( idir );
    } else {
        if( fext ) *fext = fname.substr( iext );
        if( fshort ) *fshort = fname.substr( idir, iext-idir-1 );
    }
}

int mystd::glob_file_list( const string text, 
			   const string pre, const string ext,
			   vector<string> *flong, vector<string> *fshort )
// input:
//     text:   bash-like file list [actually it works only for 1 name
//                                  which may however contain wildcards]
//     pre:    prepend this
//     ext:    append this (except if text already contains extension)
// output:
//     flong:  unix file names
//     fshort: internal (rda) file names  
{
    string globpattern;
    if (pre!=string("")) 
        globpattern = pre + string("/") + text;
    else
        globpattern = text;
    if( ext!=string("") and
        not ( (int)(text.rfind(".")) > 0 and (int)(text.rfind(".")) > (int)(text.rfind("/")) ) ) 
        globpattern += string(".") + ext;

    glob_t globbuf;
    glob(globpattern.c_str(), GLOB_TILDE, NULL, &globbuf);
    // printf( "DEBUG: text[%s] -> globpattern[%s] -> count[%i] because pos(.)=%i, pos(/)=%i\n",
    //        text.c_str(), globpattern.c_str(), globbuf.gl_pathc, text.rfind("."), text.rfind("/") );

    int i;
    for ( i=0; i<globbuf.gl_pathc; i++ ) {
        string fnam = globbuf.gl_pathv[i];
        uint ipre = fnam.rfind("/") + 1;
        uint isub = fnam.rfind(string(".")+ext) - ipre;
        if( flong ) flong->push_back(fnam);
        if( fshort) fshort->push_back(fnam.substr(ipre, isub));
    }

    globfree(&globbuf);
    return i;
}

int mystd::glob_one_file( const string text, const string pre,
                          const string ext, string *fname)
{
    vector<string> flong, fshort;
    int ret = glob_file_list(text, pre, ext, &flong, &fshort);
    if (ret>=1) *fname = flong[0];
    return ret;
}

FILE* mystd::glob_fopen(const string text, const string pre, const string ext, 
			const char* mod, string *fname)
{
    string flong;
    if (glob_one_file(text, pre, ext, &flong)!=1) return 0;
    if (fname) *fname=flong;
    return fopen(flong.c_str(), mod);
}

int mystd::freadln(ifstream *f, string *s)
{
    *s = "";
    char c;
    while (1) {
        if (!f->get(c)) return -1; // EOF
        if (c=='\n') return s->size();
        s += c;
    }
}

int mystd::freadln(FILE *fd, string *s)
{
    *s = "";
    char c;
    while (1) {
        c = fgetc(fd);
        if (c==EOF) return -1; // EOF
        if (c=='\n') return s->size();
        *s += c;
    }
}
