//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* file_out.cpp                                                           *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-, this file 2009-               *//
//* http://www.messen-und-deuten.de/frida                                  *//
//**************************************************************************//

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <map>

#include <ask_simple_value.h>

#include "mystd.h"
#include "olf.h"
#include "mem.h"
#include "file_out.h"

using namespace std;

namespace NFileOut {
    void Save_y08( FILE *file, POlo f );
    void Save_a01( FILE *file, POlo f );
    void Save_csv( FILE *file, POlo f );
    void Save_tab( FILE *file, POlo f );
}


//! Save files. Loop over file selection, and switch to format-sepcific calls.

void NFileOut::Save( string fmt, bool allow_overwrite )
{
    string outfnam;
    FILE *file;
    NOlm::IterateO iter;
    POlo f;
    while( (f=iter()) ) {
        // query output file name
        outfnam = wask( "Save as (."+fmt+")", f->name);
        if ( outfnam=="" ) return;
        f->name = outfnam;
        outfnam += "."+fmt;
        // document file save
        f->lDoc.push_back( "fs " + outfnam + " # " +
                           mystd::myasctime(mystd::justnow(),0) );
        // prevent unintended overwriting
        if( !allow_overwrite ){
            if ( (file = fopen(outfnam.c_str(), "r")) ) {
                fclose(file);
                throw "file "+outfnam+" exists, use fs! to overwrite";
            }
        }
        // save file
        if ( !(file = fopen(outfnam.c_str(), "w")) )
            throw "cannot write to file " + outfnam;
        if( fmt=="y08"){
            Save_y08( file, f );
        } else if( fmt=="a01"){
            Save_a01( file, f );
        } else if( fmt=="csv" ) {
            Save_csv( file, f );
        } else if( fmt=="tab" ) {
            Save_tab( file, f );
        } else
            throw string( "Unknown format" );
        fclose(file);
        f->as_on_disk = true;
    }
}


// Save file in format y08.

void NFileOut::Save_y08( FILE *file, POlo f )
{
    POld fd = P2D( f );
    POlc fc = P2C( f );
    string out;
    fprintf( file, "Meta:\n"
             "  format:  frida/y08 for yaml1\n"
             "  type:    %s\n",
             fd ? "generic tabular data" : "frida2 curve" );

    fprintf( file, "History:\n" );
    for ( uint i=0; i<f->lDoc.size(); i++ )
        fprintf( file, "- %s\n", yaml(f->lDoc[i]).c_str() );
    fprintf( file, "- \%s\n", yaml( out ).c_str() );

    fprintf( file, "Coord:\n" );
    fprintf( file, "  x:\n    name: %s\n    unit: %s\n", 
             yaml(f->xco.name).c_str(), yaml(f->xco.unit).c_str() );
    fprintf( file, "  y:\n    name: %s\n    unit: %s\n", 
             yaml(f->yco.name).c_str(), yaml(f->yco.unit).c_str() );
    for ( uint i=0; i<f->ZCo.size(); i++ )
        fprintf( file, "  z%u:\n    name: %s\n    unit: %s\n", 
                 i, yaml(f->ZCo[i].name).c_str(),
                 yaml(f->ZCo[i].unit).c_str() );

    fprintf( file, "Param:\n" );
    for ( uint i=0; i<f->RPar.size(); i++ )
        fprintf( file,
                 "  -\n    name:  %s\n    unit:  %s\n    value: %20.15g\n",
                 yaml(f->RPar[i].Co.name).c_str(),
                 yaml(f->RPar[i].Co.unit).c_str(),
                 f->RPar[i].val );

    if( fc ) {
        fprintf( file, "Formula: %s\n", fc->expr.c_str() );
    }

    fprintf( file, "Tables: # %u tables follow\n", f->nJ() );

    for( uint j=0; j<f->nJ(); j++ ){
        fprintf( file, "  - # table %u\n", j );
        if   ( fd ) {
            PSpec s = fd->VS(j);
            for ( uint i=0; i<f->nZ(); i++ ) 
                fprintf( file, "    z%u: %18.10g\n", i, s->z[i] );
            if( s->dy.size() ) {
                fprintf( file, "    xyd: |2 # %u entries\n", s->size() );
                for( uint i=0; i<s->size(); i++ ) 
                    fprintf( file, "      %20.15g %20.15g %20.15g\n",
                             s->x[i], s->y[i], s->dy[i] );
            } else {
                fprintf( file, "    xy: |2 # %u entries\n", s->size() );
                for( uint i=0; i<s->size(); i++ ) 
                    fprintf( file, "      %20.15g %20.15g\n",
                             s->x[i], s->y[i] );
            }
        } else {
            PCurve s = fc->VC(j);
            for ( uint i=0; i<f->nZ(); i++ ) 
                fprintf( file, "    z%u: %18.10g\n", i, s->z[i] );
            for ( uint i=0; i<fc->nPar(); i++ ) 
                fprintf( file, "    p%u: %18.10g\n", i, s->P[i] );
        }
    }
}


//! Save file in format ASCII-01.

void NFileOut::Save_a01( FILE *file, POlo f )
{
    POld fd = P2D( f );
    POlc fc = P2C( f );
    fprintf(file, "ASCII-01 JWu\n");
    if( fd )
        fprintf(file, "typ D\n");
    else
        throw string( "ASCII-01 save only for data files" );
    fprintf( file, "fil %s\n", f->name.c_str() );
    for ( uint i=0; i<f->lDoc.size(); i++ )
        fprintf( file, "doc %s\n", f->lDoc[i].c_str() );

    fprintf( file, "x   %s\n", f->xco.to_a01().c_str() );
    fprintf( file, "y   %s\n", f->yco.to_a01().c_str() );
    for ( uint i=0; i<f->ZCo.size(); i++ )
        fprintf( file, "z%-2u %s\n", i, f->ZCo[i].to_a01().c_str() );
    for ( uint i=0; i<f->RPar.size(); i++ )
        fprintf( file, "rpa %20.15g %s\n",
                 f->RPar[i].val, f->RPar[i].Co.to_a01().c_str() );
    
    fprintf( file, "#j  %u\n", f->nJ() );
    for( uint j=0; j<f->nJ(); j++ ){
        PSpec s = fd->VS(j);
        fprintf( file, "scn %u\n", j );
        fprintf( file, "Z   %u\n", fd->nZ() );
	for ( uint i=0; i<fd->nZ(); i++ ) 
            fprintf( file, "%18.10g\n", s->z[i] );
	fprintf( file, "XY  %u\n", s->size() );
        for( uint i=0; i<s->size(); i++ ) 
            fprintf( file, "%20.15g %20.15g\n", s->x[i], s->y[i] );
    }
    fprintf(file, "eof\n");
    f->as_on_disk = true;
}


//! Write data to table.

void NFileOut::Save_csv( FILE *file, POlo f )
{
    POld fd = P2D( f );
    POlc fc = P2C( f );
    if( !fd )
        throw string( "csv save only for data files" );
    for( uint j=0; j<f->nJ(); j++ ){
        for( uint i=0; i<fd->VS(j)->size(); ++i ){
            fprintf( file, "%g\t", fd->VS(j)->y[i] );
        }
        fprintf( file, "\n" );
    } 
}


//! Write data to table.

void NFileOut::Save_tab( FILE *file, POlo f )
{
    POld fd = P2D( f );
    POlc fc = P2C( f );
    if( !fd )
        throw string( "tab save only for data files" );
    for( uint j=0; j<f->nJ(); j++ ){
        for( uint iz=0; iz<f->ZCo.size(); ++iz ){
            if( iz>0 )  fprintf( file, "\t" );
            fprintf( file, "%g", fd->VS(j)->z[iz] );
        }
        fprintf( file, "\n" );
        for( uint i=0; i<fd->VS(j)->size(); ++i ){
            fprintf( file, "%g\t%g\n", fd->VS(j)->x[i], fd->VS(j)->y[i] );
        }
        fprintf( file, "\n" );
    } 
}
