//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* frida2.cpp: main loop                                                  *//
//* (C) Joachim Wuttke 1990-, C++ rewrite 2001-                            *//
//* http://www.messen-und-deuten.de/frida                                  *//
//**************************************************************************//

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <gsl/gsl_errno.h>

#include <ask_simple_value.h>
#include <readln.h>

#include "../config.h"

#include "mystd.h"
#include "olf.h"
#include "mem.h"
#include "calc.h"
#include "dualplot.h"
#include "opr.h"
#include "edif.h"
#include "func.h" // for initialization
#include "plot.h"
#include "file_in.h"
#include "file_out.h"
#include "rssm.h"
#include "manip.h"
#include "curve.h"
#include "special.h"
#include "rng.h"

using namespace std;


void my_gsl_error_handler (
    const char * reason, const char * file, int line, int gsl_errno )
{
    cout << ( "GSL error " + strg(gsl_errno) + ": " + reason + " in " +
              file + ", line " + strg(line) + "\n" );
    // throw does not work (probably because GSL is in C, not in C++)
}


int main()
{
    // ** Initializations **

    // External setup file:
    try {
        NRead::exemac("\\i " CONFDIR "/frida.ini");
    } catch ( string& s ) {
        cerr << s << endl;
    }

    // Math:
    gsl_set_error_handler( my_gsl_error_handler );
    NRNG::init();
    NFunctions::initialize();

    // Plot windows:
    vector<CPlot*> Plots;
    uint nPlot = 0; // shorthand for Plots.size()
    Plots.push_back( new CPlot( nPlot++, false, false ) );
    Plots.push_back( new CPlot( nPlot++, true,  false ) );
    Plots.push_back( new CPlot( nPlot++, false, true ) );
    Plots.push_back( new CPlot( nPlot++, true,  true ) );
    uint i, iPlot = 0; // current plot window

    // Time spent in frida, for optimizing numerical routines:
    clock_t clock_start = clock();
    clock_t clock_cmd_i;
    double time_cmd=0;


    // ** Main command loop **

    while(1) {

        try{
            // Get a word from the user:
            string prompt =
                (NOlm::FSel.size() ? NOlm::FSel.str() : "?") + " > ";
            string cmd = wask( prompt.c_str() );

            // If it's a list, then make it the default and request next word:
            if        ( cmd=="-" ) {
                NOlm::SelPop();
                continue;
            } else if ( cmd.find_first_of("*0123456789")==0 ) {
                CList newSel( cmd, 0, NOlm::MOM.size()-1 );
                NOlm::SelPush( newSel );
                continue;
            }

            // Extract overwrite marker:
            NOlm::overwrite = 0;
            if ( cmd[0]=='=' ) {
                NOlm::overwrite = 1;
                cmd.erase(0,1);
                if( cmd=="" )
                    throw string( "overwrite marker = must immediately"
                                  " preceed a command" );
            }

            // All other commands:
            NOlm::FSelNew = CList();
            clock_cmd_i = clock();
			
            if (cmd == "h" || cmd == "?") {
                cout <<
                    "This is Frida, version " VERSION "\n"
                    "Plain commands:\n"
                    "  a    add to plot\n"
                    "  p    plot\n"
                    "  t    show execution time (for development)\n"
                    "  x    execute shell command\n"
                    "  .    calculator mode (or evaluate given expression)\n"
                    "  quit terminate Frida session\n"
                    "Command groups (type initial letter for help):\n"
                    "  c    curves and fits\n"
                    "  d    directory of online files\n"
                    "  e    edit coordinate names and more\n"
                    "  g    graphics setup\n"
                    "  f    file operations\n"
                    "  m    manipulations (reorganisation of data)\n"
                    "  o    operations (arithmetic operations on data)\n"
                    "  r    raw data read in (currently only for SPHERES)\n"
                    "  _    special / temporary commands\n"
                    "More help:\n"
                    "  \\?   help on meta commands and macros\n"
                    "  hf   math operators and functions\n"
                    ;

            } else if (cmd == "hf") {
                cout << NFunctions::list();

            } else if (cmd == "c") {
                cout << "Curves:\n"
                    "  cc   create\n"
                    "  cca  create artificial curve (no data reference)\n"
                    "Basic operations:\n"
                    "  cm   modify function\n"
                    "  cd   set data file to fit\n"
                    "  cv   set file to convolute with (cv-: no convolution)\n"
                    "  cr   set fit range restriction\n"
                    "  ci   extract parameter, calculate integral, ...\n"
                    "  cp   print parameters\n"
                    "Fit to data:\n"
                    "  cf   fit\n"
                    "  cfs  fit, allow slow convolution\n"
                    "  cx   toggle fixed (also: cx+ cx-)\n"
                    "  cwc  weighing: constant\n"
                    "  cwl  weighing: logarithmic\n"
                    "  cwv  weighing: reciprocal variance (default)\n"
                    "Fit tuning parameters:\n"
                    "  cs?  show parameters\n"
                    "  csf  set gradient step factor\n"
                    "  cse  set sqrt of differentiation step\n"
                    "  csc  set number of calls\n"
                    "  csp  set print frequency\n"
                    "  csm  set fit metric\n"
                    "  cst  set maximum duration of fits\n"
                    "Tuning parameters for integration:\n"
                    "  cn?  show parameters\n"
                    "  cnm  set integration mode\n"
                    "  cna  set eps abs\n"
                    "  cnr  set eps rel\n"
                    "Tuning parameters for convolution:\n"
                    "  cv?  show parameters\n"
                    "  cvm  set integration mode\n"
                    "  cva  set eps abs\n"
                    "  cvr  set eps rel\n"
                    ;

            } else if (cmd == "cc" ) {
                NCurveFile::CreateFitcurve();
            } else if (cmd == "cca" ) {
                NCurveFile::CreateFreecurve();

            } else if (cmd == "cm" ) {
                NCurveFile::ChangeExpr();

            } else if (cmd == "cd" || cmd == "cv" || cmd == "cv-" ) {
                NCurveFile::SetFileReference( cmd.substr(1) );

            } else if (cmd == "cr" ) {
                NCurveFile::ChangeRange();

            } else if (cmd == "ci") {
                NCurveFile::IntegralProperty();
            } else if (cmd == "cp") {
                NCurveFile::ShowPar();

            } else if (cmd == "cf") {
                NCurveFile::Fit( false );
            } else if (cmd == "cfs") {
                NCurveFile::Fit( true );
            } else if (cmd == "cx") {
                NCurveFile::SetFixed();
            } else if (cmd.substr(0,2) == "cw") {
                NCurveFile::SetProperties( cmd.substr(1) );

            } else if (cmd == "c_cd") {
                NCurveFile::CumulativeDistribution();

            } else if (cmd.substr(0,2) == "cs") {
                NCurveFile::SetFitTuningPars(cmd.substr(2));

            } else if (cmd.substr(0,2) == "cn") {
                NCurveFile::SetIntTuningPars(cmd.substr(2));

            } else if (cmd.substr(0,2) == "cv") {
                NCurveFile::SetConvTuningPars(cmd.substr(2));


            } else if ( cmd == "d" ) {
                cout << "Show list:\n"
                    "  df   on-line files\n"
                    "  dz   z-values and spectral ranges\n"
                    "  dp   x|y-values\n"
                    "  dy   y-values\n"
                    "  dn   numeric parameters\n"
                    "  dc   coordinate names and units\n"
                    "  dd   doc lines\n"
                    ;

            } else if (cmd == "df") {
                NEdif::ShowFiles();
            } else if (cmd == "dz") {
                NEdif::ShowSpectra();
            } else if (cmd == "dp" ) {
                NOperate::Show( "xyd" );
            } else if (cmd == "dy" ) {
                NOperate::Show( "y" );
            } else if (cmd == "dn") {
                NEdif::ShowNumpar();
            } else if (cmd == "dc") {
                NEdif::ShowCoord();
            } else if (cmd == "dd") {
                NEdif::ShowDoc();

            } else if ( cmd == "e" ) {
                cout << "Edit parameters:\n"
                    "  ef         file name\n"
                    "  ec<coord>  name and unit of <coord>=x,y,z0,..\n"
                    "  ed         documentation lines\n"
                    "  en         numeric parameters\n"
                    "  en+        - add\n"
                    "  en-        - delete\n"
                    ;

            } else if (cmd.substr(0,2) == "ec") {
                NEdif::EditCoord( cmd.substr(2) );
            } else if (cmd == "ed") {
                NEdif::EditDoc();
            } else if (cmd == "ef") {
                NEdif::EditFNam();
            } else if (cmd == "en") {
                // maybe, this should be split into ecn<nr> and on<nr>
                NEdif::EditNumpar();
            } else if (cmd == "en+") {
                NEdif::AddNumpar();
            } else if (cmd == "en-") {
                NEdif::DelNumpar();


            } else if (cmd == "f") {
                cout << "File operations:\n"
                    "  fl   load from disk\n"
                    "  fs   save to disk\n"
                    "  fs!  save to disk, allow overwrite\n"
                    "  fsa  save to disk (old a01 format)\n"
                    "  fe1  export (spectrum -> tab-separated line)\n"
                    "  fe2  export (as tab)\n"
                    "  fc   copy\n"
                    "  fx   exchange (copy, then delete source)\n"
                    "  fdel delete\n"
                    "  fm   make\n"
                    "  fr   read in\n"
                    "  ft(h|v|c)[m][s]  read tab\n"
                    "       h horizontal (only y values, one line per spectrum)\n"
                    "       v vertical (one x-y pair per line)\n"
                    "       c choose vertical columns\n"
                    "       m multi-block (blocks begin with z line)\n"
                    "       s from script\n"
                    ;

            } else if (cmd == "fl") {
                NFileIn::Load();
            } else if (cmd == "fs") {
                NFileOut::Save( "y08" );
            } else if (cmd == "fs!") {
                NFileOut::Save( "y08", true );
            } else if (cmd == "fsa") {
                NFileOut::Save( "a01" );
            } else if (cmd == "fe1") {
                NFileOut::Save( "csv" );
            } else if (cmd == "fe2") {
                NFileOut::Save( "tab" );
            } else if (cmd == "fc") {
                NOlm::OlfCopy();
            } else if (cmd == "fx") {
                NOlm::OlfExchange();
            } else if (cmd == "fdel") {
                NOlm::OlfDel();
            } else if (cmd == "fm") {
                NEdif::MakeGrid();
            } else if (cmd == "fr") {
                NEdif::ReadIn();
            } else if (cmd.substr(0,2) == "ft") {
                NEdif::ReadTab( cmd.substr(2) );


            } else if (cmd == "g") {
                cout <<
                    "Plot:\n"
                    "  p    plot\n"
                    "  pp   plot with new autoranges\n"
                    "  a    add to plot\n"
                    "  gd   graphic dialog (feed commands to gnuplot)\n"
                    "Save for print:\n"
                    "  gp   plot to complete .ps file\n"
                    "  gf   plot to short .psX file\n"
                    "Change window:\n"
                    "  gw   list of plot windows\n"
                    "  g<n> switch to plot window <n>\n"
                    "Set axes:\n"
                    "  gx   set x range\n"
                    "  gy   set x range\n"
                    "  ga   auto-ranges\n"
                    "  gxa  x auto-range\n"
                    "  gya  y auto-range\n"
                    "  gxl  log x (toggle; or use gxl+ gxl-)\n"
                    "  gyl  log y\n"
                    "  gxf  force x into frame (toggle; or use gxf+ gxf-)\n"
                    "  gyf  force x into frame\n"
                    "Other setup:\n"
                    "  ge   plot error bars (toggle, or ge+ ge-)\n"
                    "  gnp  set maximum number of points\n"
                    ;

            } else if ( cmd == "gx" ) {
                Plots[iPlot]->X.ask_and_set( "Plot x range" );
            } else if ( cmd == "gy" ) {
                Plots[iPlot]->Y.ask_and_set( "Plot y range" );
            } else if ( cmd == "ga" ) {
                Plots[iPlot]->X.setAuto();
                Plots[iPlot]->Y.setAuto();
            } else if ( cmd == "gxa" ) {
                Plots[iPlot]->X.setAuto();
            } else if ( cmd == "gya" ) {
                Plots[iPlot]->Y.setAuto();
            } else if ( cmd.substr(0,2) == "gx" ||
                        cmd.substr(0,2) == "gy" ||
                        cmd.substr(0,2) == "ge" ) {
                Plots[iPlot]->setAux( cmd );
            } else if ( cmd=="gnp" ) {
                Plots[iPlot]->maxpoints =
                    iask( "Max # points in plot", Plots[iPlot]->maxpoints );
            } else if (cmd == "gp" || cmd == "gf" ) {
                string ps_outdir = NRead::wofmac( "\\psdir" );
                string ps_head = NRead::wofmac( "\\pshead" );
                string ps_dict;
                if ( cmd=="gp" )
                    ps_dict = NRead::wofmac( "\\psdict" );
                else
                    ps_dict = "";
                Plots[iPlot]->writePostscript( ps_outdir, ps_head, ps_dict );
            } else if ( cmd == "gw" ) {
                for( uint i=0; i<nPlot; ++i ){
                    printf( " %c %1i %s\n", i==iPlot ? '*' : ' ', i,
                            Plots[i]->info().c_str() );
                }
            } else if ( sscanf( cmd.c_str(), "g%i", &i )==1 ) {
                if( i<nPlot )
                    iPlot = i;
                else if ( i==nPlot ){
                    iPlot = nPlot;
                    Plots.push_back( new CPlot( nPlot++, false, false ) );
                } else
                    throw string( "invalid graphic window number" );
            } else if (cmd == "gd") { // graph dialog
                string cmd;
                cout << "entering mygnuplot - to leave, type q\n";
                while(1) {
                    cmd = sask("mygnuplot> ");
                    if (cmd.substr(0,1)=="q")
                        break;
                    Plots[iPlot]->gp_write(cmd);
                }
            } else if (cmd == "p") {
                NPlot::Plot( Plots[iPlot], 0);
            } else if (cmd == "pp") {
                Plots[iPlot]->X.setAuto();
                Plots[iPlot]->Y.setAuto();
                NPlot::Plot( Plots[iPlot], 0 );
            } else if (cmd == "a") {
                NPlot::Plot( Plots[iPlot], 1 );


            } else if (cmd == "m") {
                cout << "Meta:\n"
                    "  m/    protect spectra from forthcoming operations\n"
                    "  m/-   lift protections\n"
                    "Generic manipulations (expression driven):\n"
                    "  md    generic delete\n"
                    "  mr    generic retain\n"
                    "Per point:\n"
                    "  mpd   delete\n"
                    "  mpr   retain\n"
                    "  mpa   average (=bin), list controlled\n"
                    "  mpaf  average, reduce number of points by fixed factor\n"
                    "  mpaea average according to error bound (absolute)\n"
                    "  mpaer average according to error bound (relative)\n"
                    "  mpo   sort\n"
                    "  mpq   average when x equal\n"
                    "  mpsym symmetrize: map -x -> x, then average\n"
                    "Per spectrum:\n"
                    "  msd   delete\n"
                    "  msr   retain\n"
                    "  msa   average\n"
                    "  msb   break into files\n"
                    "  msj   join\n"
                    "  ms*   spawn spectra\n"
                    "  msx   z_max <-> x\n"
                    "  mso   sort by expression\n"
                    "  msoz  sort by z-values\n"
                    "  mzx   exchange / rotate z coordinates\n"
                    "  mz-   delete z coordinate\n"
                    "Per file:\n"
                    "  mfj   merge (\"join\")\n"
                    "  mfj+  merge, adding z=number-of-input-file\n"
                    "  mfjp  merge pointwise\n"
                    "  mn+   add num par\n"
                    "  mn-   remove num par\n"
                    "Histogram operations:\n"
                    "  mhm   bin data AUSSER BETRIEB\n" // TODO
                    ;

            } else if (cmd == "m/") {
                NManip::ProtectSpecs( true );
            } else if (cmd == "m/-") {
                NManip::ProtectSpecs( false );
            } else if (cmd == "md") {
                NOperate::Select(1);
            } else if (cmd == "mr") {
                NOperate::Select(0);
            } else if (cmd == "mpd") {
                NManip::PtsSelect( true );
            } else if (cmd == "mpr") {
                NManip::PtsSelect( false );
            } else if (cmd == "mpa") {
                NManip::PtsAvge();
            } else if (cmd == "mpaf") {
                NManip::PtsAvgeFixed();
            } else if (cmd.substr(0,4) == "mpae") {
                NManip::PtsAvgeByErr( cmd.substr(4) );
            } else if (cmd == "mpo") {
                NManip::PtsSort();
            } else if (cmd == "mpq") {
                NManip::PtsAvgeEq();
            } else if (cmd == "mpsym") {
                NManip::PtsSymmetrize();
            } else if (cmd == "msd") {
                NManip::ScaSelect( true );
            } else if (cmd == "msr") {
                NManip::ScaSelect( false );
            } else if (cmd == "msa") {
                NManip::ScaAvge();
            } else if (cmd == "msb") {
                NManip::ScaBreak();
            } else if (cmd == "msj") {
                NManip::ScaJoin();
            } else if (cmd == "ms*") {
                NManip::ScaSpawn();
            } else if (cmd == "msx") {
                NManip::ScaExch();
            } else if (cmd == "mzx") {
                NManip::ZExchange();
            } else if (cmd == "mz-") {
                NEdif::ShowCoordZ();
                NManip::ZDelete();
            } else if (cmd == "mso") {
                NManip::ScaSortByExpr();
            } else if (cmd == "msoz") {
                NManip::ScaSortByZ();
            } else if (cmd == "mfjp") {
                NManip::FilMergePointwise();
            } else if (cmd.substr(0,3) == "mfj") {
                NManip::FilMerge( cmd.substr(3) );
//            } else if (cmd == "mhm") {
//                NManip::HistoMake();


            } else if (cmd == "o") {
                cout << "Operate:\n"
                    "  ox       on x\n"
                    "  oy       on y\n"
                    "  oz0,..   on z0,..\n"
                    "  oz+      add new z value\n"
                    "  op0,..   on p0,.. (fit parameter)\n"
                    "  op       on all fit parameters\n"
                    "  ofi      functional integrate\n"
                    "  ofd      functional derivate\n"
                    "  ofac     functional autocorrelate\n"
                    "  oi       rank-reducing (""integral"") operations\n"
                    "  oio      ditto with oldstyle menu\n"
                    "  oixy     choose two columns, save them as x and y\n"
                    ;

            } else if (cmd == "op") {
                NCurveFile::SetAllPars();
            } else if (cmd[0]=='o' && strchr("xydzp", cmd[1])) {
                NOperate::Pointwise( cmd.substr(1) );
            } else if (cmd.substr(0,2) == "of") {
                NOperate::Functional( cmd.substr(2) );
            } else if (cmd == "oi") {
                NOperate::Integral();
            } else if (cmd == "oixy") {
                NOperate::IntXY();
            } else if (cmd == "oio") {
                NIntOld::Opr();


            } else if (cmd == "r") {
                cout << "read raw data from SPHERES:\n"
                    "  ry   load energy spectra\n"
                    "  ry2  load also spectra for open state\n"
                    "  ry4  load also chopper histograms\n"
                    "  ry8  load spectra for two directions separately\n"
                    "  ryd  load only spectra for two directions\n"
                    "  rs   load series of energy spectra\n"
                    "  rs2  load also spectra for open state\n"
                    ;

            } else if (cmd == "ry" ) {
                NRSSM::ReadSpec( 0 );
            } else if (cmd == "ry2") {
                NRSSM::ReadSpec( 1 );
            } else if (cmd == "ry4") {
                NRSSM::ReadSpec( 3 );
            } else if (cmd == "ry8") {
                NRSSM::ReadSpec( 7 );
            } else if (cmd == "ryd") {
                NRSSM::ReadSpec( 12 );
            } else if (cmd == "rs") {
                NRSSM::ReadSeries( 0 );
            } else if (cmd == "rs2") {
                NRSSM::ReadSeries( 1 );


            } else if (cmd == "t") {
                double time_total =
                    ((double) (clock() - clock_start)) / CLOCKS_PER_SEC;
                printf( "total frida CPU time now %g\n", time_total );
                printf( "last command took %g\n", time_cmd );


            } else if (cmd == "x") {
                string sh_cmd;
                sh_cmd = sask("shell command");		
                mystd::system( sh_cmd );


            } else if (cmd == ".") {
                NCalc::Calculator();
           

            } else if (cmd == "_") {
                cout << "Special programs:\n"
                    "  _evv     eigenvalues and eigenvectors\n"
                    "  _cc      correlations (y vs y')\n"
                    "  _cd      differentiate correlations\n"
                    "  _fc      cosine Fourier transform\n"
                    "  _t       test (may change at any moment)\n"
//                    "  _ps      power spectrum [disabled]\n"
                    ;


            } else if (cmd == "_evv") {
                NSpecial::Eigen();
            } else if (cmd == "_cc") {
                NSpecial::CalcCorrel();
            } else if (cmd == "_cd") {
                NSpecial::CalcCorrelDiff();

            } else if (cmd == "_fc") {
                NSpecial::FourierCosine();

            } else if (cmd == "_t") {
                NSpecial::Test();

            } else if ( cmd == "qui" || cmd=="quit" ) {
                exit(0);


            } else if ( cmd == "" ) {
                ;

            } else {
                throw "invalid command '" + cmd + "'";
            }

            // If there are new files, make them new default
            if( !NOlm::overwrite && NOlm::FSelNew.size()>0 )
                NOlm::SelPush( NOlm::FSelNew );

        } catch( string& s ) {
            cerr << s << endl;
            NRead::clear();
        } catch( const char* s ) {
            cerr << s << endl;
            NRead::clear();
        } catch( ... ) {
            cerr << "catched invalid exception\n";
            NRead::clear();
        }
        time_cmd = ((double) (clock() - clock_cmd_i)) / CLOCKS_PER_SEC;
    } // end main command loop
}
