//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* func.cpp: elementary functions for use in generic expressions          *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://www.messen-und-deuten.de/frida                                  *//
//**************************************************************************//

#include <math.h>
#include <iostream>
#include <map>
#include <gsl/gsl_sf.h>
#include <kww.h>
#include <ask_simple_value.h> //for find

#include "mystd.h"
#include "rng.h"
#include "func.h"
#include "coord.h"

const double twopi = 8*atan(1.);

//**************************************************************************//
//*   Functions of one argument                                            *//
//**************************************************************************//

double func_not(double v) { return !v; }
double func_neg(double v) { return -v; }
void deri_neg(double& r, double& dr, double v, double dv)
{ r=-v; dr=dv; }

double func_ln(double v) { return v>0 ? log(v) : 0; }
void deri_ln(double& r, double& dr, double v, double dv)
{ if(v>0) { r=log(v); dr=dv/v; } else { r=0; dr=0; } }

double func_lg(double v) { return v>0 ? log10(v) : 0; }
void deri_lg(double& r, double& dr, double v, double dv)
{ if(v>0) { r=log10(v); dr=dv/v/log(10); } else { r=0; dr=0; } }
	
double func_sqrt(double v) { return v>=0 ? sqrt(v) : 0; }
void deri_sqrt(double& r, double& dr, double v, double dv)
{ if(v>0) { r=sqrt(v); dr=dv/2/r; } else { r=0; dr=0; } } 

double func_abs(double v) { return fabs(v); }
void deri_abs(double& r, double& dr, double v, double dv)
{ r=fabs(v); dr=dv; }

double func_exp(double v) { return exp(v); }
void deri_exp(double& r, double& dr, double v, double dv)
{ r=exp(v); dr=dv*r; }
	
double func_sin(double v) { return sin(v); }
double func_cos(double v) { return cos(v); }
double func_tan(double v) { return tan(v); }
double func_cot(double v) { double t=tan(v); return t==0?0:1/t;}

double func_sind(double v) { return sin(v*twopi/360.); }
double func_cosd(double v) { return cos(v*twopi/360.); }
double func_tand(double v) { return tan(v*twopi/360.); }
double func_cotd(double v) { double t=tan(v*twopi/360.);  return t==0?0:1/t; }
	
double func_asin(double v) { return (-1<=v && v<=1) ? asin(v) : 0; }
double func_acos(double v) { return (-1<=v && v<=1) ? acos(v) : 0; }
double func_atan(double v) { return atan(v); }
double func_acot(double v) { return v!=0 ? atan(1/v) : twopi/4; }

double func_asind(double v) { return (-1<=v && v<=1) ? 360./twopi*asin(v) : 0; }
double func_acosd(double v) { return (-1<=v && v<=1) ? 360./twopi*acos(v) : 0; }
double func_atand(double v) { return 360./twopi*atan(v); }
double func_acotd(double v) { return v!=0 ? 360./twopi*atan(1/v) : 90; }

double func_sinh(double v) { return sinh(v); }
double func_cosh(double v) { return cosh(v); }
double func_tanh(double v) { return tanh(v); }
double func_coth(double v) { double t=tanh(v); return t==0?0:1/t;}

double func_gamma(double v) { return v>0 ? gsl_sf_gamma(v) : 0; } 
// erweiterungsfhig (v<0 aber nichtganzzahlig ?)

double func_erfP(double v) { return 1-gsl_sf_erf_Q(v); }
double func_erfQ(double v) { return gsl_sf_erf_Q(v); } 
double func_erf(double v) { return gsl_sf_erf(v); } 
double func_erfc(double v) { return gsl_sf_erfc(v); } 

double func_sinc(double v) { return v==0 ? 1 : sin(v)/v; }

double func_ceil(double v)  { return ceil(v); }
double func_floor(double v) { return floor(v); }
double func_nint(double v)  { return round(v); }

//**************************************************************************//
//*   Functions of two arguments                                           *//
//**************************************************************************//

double func_pm(double v, double a) { return v; }
void deri_pm(double& r, double& dr, double v, double dv, double a, double da)
{ r=v; dr=a; }

double func_add(double v, double a) { return v+a; }
void deri_add(double& r, double& dr, double v, double dv, double a, double da)
{ r=v+a; dr=sqrt(dv*dv+da*da); }

double func_sub(double v, double a) { return v-a; }
void deri_sub(double& r, double& dr, double v, double dv, double a, double da)
{ r=v-a; dr=sqrt(dv*dv+da*da); }

double func_mul(double v, double a) { return v*a; }
void deri_mul(double& r, double& dr, double v, double dv, double a, double da)
{ r=v*a; dr=sqrt(SQR(v*da)+SQR(a*dv)); }

double func_div(double v, double a) { return a!=0 ? v/a : 0; }
void deri_div(double& r, double& dr, double v, double dv, double a, double da)
{ if(a==0) { r=0; dr=0; } else { r=v/a; dr=sqrt(SQR(dv/a)+SQR(v*da/a/a)); } }

double func_pow(double v, double a)
{ return ( v>0 || (v==0&&a>0) || a==(int)a ) ? pow(v,a) : 0; }
void deri_pow(double& r, double& dr, double v, double dv, double a, double da)
{
    if ( v>0 || (v==0&&a>0) || a==(int)a ) {
        r = pow(v,a); dr = sqrt( SQR(a*r/v) + SQR(log(v)*a) );
    } else {
        r = 0; dr = 0;
    }
}

double func_eq (double v, double a) { return v == a; }
double func_ne (double v, double a) { return v != a; }
double func_gt (double v, double a) { return v > a; }
double func_ge (double v, double a) { return v >= a; }
double func_lt (double v, double a) { return v < a; }
double func_le (double v, double a) { return v <= a; }
double func_and(double v, double a) { return v && a; }
double func_or (double v, double a) { return v || a; }
double func_xor(double v, double a) { return (bool) v ^ (bool) a; }

double func_idiv(double v, double a) { return a!=0 ? ((int)v)/((int)a) : 0; }
double func_mod(double v, double a) { return fmod(v,a); }
double func_over(double v, double a) {
    return  ( 0<=v && 0<=a && a<=v && v <= GSL_SF_GAMMA_XMAX ) ?
        gsl_sf_gamma(v+1)/gsl_sf_gamma(a+1)/gsl_sf_gamma(v-a+1) :
        0; }

double func_min (double v, double a) { return v<a ? v : a; }
double func_max (double v, double a) { return v>a ? v : a; }

double func_gauss (double x, double s) {
    return s==0 ? 0 : exp(-x*x/(2*s*s))/sqrt(twopi)/s; }
double func_gaussnn (double x, double s) { // Gauss not normalized
    return s==0 ? 0 : exp(-x*x/(2*s*s)); }
double func_cauchy (double x, double s) {
    return s==0 ? 0 : 2*fabs(s)/twopi/(x*x+s*s); }

double func_kwwc( double v, double a ){
    if( a<0.1 || a>2.0 ) return 0;
    return kwwcf( (float)v, (float)a ); }
double func_kwws( double v, double a ){
    if( a<0.1 || a>2.0 ) return 0;
    return kwwsf( (float)v, (float)a ); }
double func_skww( double w, double _b, double _wc ){
    // KWW S(q,w) with round-off at wc
    static double b=0, wc=0, sig2=0, amp=0;
    if( _b<0.1 || _b>2.0 || _wc<=0 ) return 0;
    if( _b!=b || _wc!=wc ){
        b = _b;
        wc = _wc;
        sig2 = - SQR(wc)*2e-4 /
            ( log( (double) kwwcf( (float)wc*(1+1e-4), (float)b ) ) -
              log( (double) kwwcf( (float)wc*(1-1e-4), (float)b ) ) );
        amp =  (double) kwwcf( (float)wc, (float)b ) * exp( SQR(wc)/2/sig2 );
    }
    if( fabs(w)>=wc )
        return (double) kwwcf( (float)w, (float)b );
    else
        return amp * exp( -SQR(w)/2/sig2 );
}

double func_ran (double v, double a) { 
    // a random number between v and a :
    return func_min(v,a) + (func_max(v,a)-func_min(v,a))*NRNG::uniform(); }

// Havriliak-Negami: unchecked formulae from disreputable source:

double aux_phi_havneg (double v, double a){
    return atan( pow(v,a)*sin(M_PI*a/2)/(1+pow(v,a)*cos(M_PI*a/2)) ); }

double func_re_havneg (double v, double a, double g) {
    return pow( 1 + 2*pow(v,a)*cos(M_PI*a/2)+pow(v,2*a), -g/2 ) *
        cos( g*aux_phi_havneg( v, a ) ); }

double func_im_havneg (double v, double a, double g) {
    return pow( 1 + 2*pow(v,a)*cos(M_PI*a/2)+pow(v,2*a), -g/2 ) *
        sin( g*aux_phi_havneg( v, a ) ); }

//**************************************************************************//
//*   Functions of three arguments                                         *//
//**************************************************************************//

double func_cond(double v, double a, double b) { return v ? a : b; }
void deri_cond(double& r, double& dr, double v, double dv,
                 double a, double da, double b, double db)
{ if(v) { r=a; dr=da; } else { r=b; dr=db; } }

double func_q4w( double hbarw, double Ei, double twotheta) { 
    // calculate Q in Angstroem-1 when hbar*w and E_i are given in meV
    if( hbarw>Ei ) return -1;
    return sqrt( (1/2.07212349) *
            ( 2*Ei-hbarw-2*sqrt(Ei*(Ei-hbarw))*cos(twotheta*twopi/360) ) );
}

double func_cauchy2 (double x, double p, double w) {
    return w==0 ? 0 : 2*w/twopi*(1.0/(SQR(x-p)+SQR(w))+1.0/(SQR(x+p)+SQR(w)));
}

//**************************************************************************//
//*   Coordinate concatenation                                             *//
//**************************************************************************//

CCoord CFunc::coord( CCoord *co1 ) const
{
    CCoord co = *co1;
    if (txt=="0-") {
        co.name = "-" + co1->name;
    } else if (txt=="ln") {
        co.name = "ln " + co1->name;
    } else if (txt=="lg") {
        co.name = "lg " + co1->name;
    } else if (txt=="sqrt") {
        co.name = "sqrt (" + co1->name + ")";
        co.unit = co1->unit=="" ? "" : "(" + co1->unit + ")^1/2";	
    } else {
        co.name = txt + "(" + co1->name + ")";        
    }
    return co;
}

CCoord CFunc::coord( class CCoord *co1, class CCoord *co2 ) const
{
    CCoord co = *co1;

    if ( txt=="+" || txt=="-" ) {
        if( co1->name==co2->name ){
            if( co1->unit!=co2->unit )
                throw string( "same coordinate name and different units" 
                              " in additive operation" );
            co = *co1;
        } else {
            co.name = co1->name + txt + co2->name;
            if ( co1->unit=="" )
                co.unit = "";
            else if ( co2->unit!="" && co2->unit!=co1->unit) {
                cout << "? different units " << co1->unit << " and " <<
                    co2->unit << " in additive operation\n";
                co.unit = "";
            } else
                co.unit = co1->unit;
        }
    } else if ( txt=="*" || txt=="/" ) {
        co.name = co1->name + txt + co2->name;
        if     ( co1->unit!="" && co2->unit!="" )
            co.unit = co1->unit + txt + co2->unit;
        else if( co1->unit!="" )
            co.unit = co1->unit;
        else if( co2->unit!="" )
            co.unit = txt=="*" ? co2->unit : "1/(" + co2->unit + ")";
        else
            co.unit = "";
    } else if ( txt=="^" ) {
        if (co2->unit != "") {
            cout << "? exponent is not dimensionless\n";
        }
        co.name = co1->name + "^" + co2->name;
        co.unit = co1->unit=="" ? "" : ( co1->unit + "^<val>" );
    }
    return co;
}

CCoord CFunc::coord( class CCoord *co1, class CCoord *co2, class CCoord *co3 )
    const
{
    CCoord co = *co2;
    return co;
}

//**************************************************************************//
//*   Function registration and retrieval                                  *//
//**************************************************************************//

namespace NFunctions { // internals:
    map<string,CFunc> fmap; // unsorted hash, for expression evaluation
    vector<CFunc> flist;    // sorted array, for help text
};

CFunc* NFunctions::find( string key )
{
    map<string,CFunc>::iterator pos;
    if ((pos = fmap.find(key)) == (fmap.end())) return 0;
    return &(pos->second);
}

string NFunctions::list()
{

    // this implementation works only, if flist is strictly ordered:
    // first operators by precedence, then functions.

    string ret = "";
    int prec = -1;
    ret += "Operators by precedence:";
    for( uint i=0; i<flist.size(); ++i ){
        const CFunc *f;
        f = &(flist[i]);
        if( f->prec ) { // operators
            if( f->prec!=prec )
                ret += "\n ";
            ret += " " + f->txt;
        } else { // functions
            if( f->prec!=prec )
                ret += "\nFunctions (number of arguments):\n";
            ret += "  " + f->txt + " (" + strg(f->narg) + ")\n";
        }
        prec = f->prec;
    }
    return ret;
}

void CFunc::register_me() const
{
    NFunctions::fmap.insert( make_pair( txt, *this ) );
    NFunctions::flist.push_back( *this );
}

void NFunctions::initialize(void) {

// operators by precedence (as in xax_yacc.ypp):
    CFunc( "+-",  func_pm,   deri_pm,    1   ).register_me();
    CFunc( "^",   func_pow,  deri_pow,   2   ).register_me();
    CFunc( "!",   func_not,  0,          3   ).register_me();
    CFunc( "neg", func_neg,  deri_neg,   4   ).register_me();
    CFunc( "over",func_over, 0,          5   ).register_me();
    CFunc( "*",   func_mul,  deri_mul,   6   ).register_me();
    CFunc( "/",   func_div,  deri_div,   6   ).register_me();
    CFunc( "mod", func_mod,  0,          6   ).register_me();
    CFunc( "div", func_idiv, 0,          6   ).register_me();
    CFunc( "+",   func_add,  deri_add,   7   ).register_me();
    CFunc( "-",   func_sub,  deri_sub,   7   ).register_me();
    CFunc( ">",   func_gt,   0,          8   ).register_me();
    CFunc( ">=",  func_ge,   0,          8   ).register_me();
    CFunc( "<",   func_lt,   0,          8   ).register_me();
    CFunc( "<=",  func_le,   0,          8   ).register_me();
    CFunc( "==",  func_eq,   0,          9   ).register_me();
    CFunc( "!=",  func_ne,   0,          9   ).register_me();
    CFunc( "xor", func_xor,  0,         10   ).register_me();
    CFunc( "&&",  func_and,  0,         11   ).register_me();
    CFunc( "||",  func_or ,  0,         12   ).register_me();
    CFunc( "?:",  func_cond, deri_cond, 13 ).register_me();

// f( 1 arg )
    CFunc( "ln",   func_ln,    deri_ln     ).register_me();
    CFunc( "lg",   func_lg,    deri_lg     ).register_me();
    CFunc( "sqrt", func_sqrt,  deri_sqrt   ).register_me();
    CFunc( "abs",  func_abs,   deri_abs    ).register_me();
    CFunc( "exp",  func_exp,   deri_exp    ).register_me();
    CFunc( "sin",  func_sin    ).register_me();
    CFunc( "cos",  func_cos    ).register_me();
    CFunc( "tan",  func_tan    ).register_me();
    CFunc( "cot",  func_cot    ).register_me();
    CFunc( "sind", func_sind   ).register_me();
    CFunc( "cosd", func_cosd   ).register_me();
    CFunc( "tand", func_tand   ).register_me();
    CFunc( "cotd", func_cotd   ).register_me();
    CFunc( "asin", func_asin   ).register_me();
    CFunc( "acos", func_acos   ).register_me();
    CFunc( "atan", func_atan   ).register_me();
    CFunc( "acot", func_acot   ).register_me();
    CFunc( "asind",func_asind  ).register_me();
    CFunc( "acosd",func_acosd  ).register_me();
    CFunc( "atand",func_atand  ).register_me();
    CFunc( "acotd",func_acotd  ).register_me();
    CFunc( "sinh", func_sinh   ).register_me();
    CFunc( "cosh", func_cosh   ).register_me();
    CFunc( "tanh", func_tanh   ).register_me();
    CFunc( "coth", func_coth   ).register_me();
    CFunc( "gamma",func_gamma  ).register_me();
    CFunc( "erfP", func_erfP   ).register_me();
    CFunc( "erfQ", func_erfQ   ).register_me();
    CFunc( "erf",  func_erf    ).register_me();
    CFunc( "erfc", func_erfc   ).register_me();
    CFunc( "sinc", func_sinc   ).register_me();
    CFunc( "ceil", func_ceil   ).register_me();
    CFunc( "floor",func_floor  ).register_me();
    CFunc( "nint", func_nint   ).register_me();

// f(2 args)
    CFunc( "min",  func_min    ).register_me();
    CFunc( "max",  func_max    ).register_me();
    CFunc( "ran",  func_ran    ).register_me();

    CFunc( "kwwc", func_kwwc   ).register_me();
    CFunc( "kwws", func_kwws   ).register_me();

    CFunc( "gauss",   func_gauss).register_me();
    CFunc( "gnn",     func_gaussnn ).register_me();
    CFunc( "cauchy",  func_cauchy ).register_me();

// f(3 args)
    CFunc( "q4w",      func_q4w    ).register_me();
    CFunc( "cauchy2",  func_cauchy2 ).register_me();
    CFunc( "skww",     func_skww    ).register_me();
    CFunc( "rehavneg", func_re_havneg    ).register_me();
    CFunc( "imhavneg", func_im_havneg    ).register_me();
}
