//**************************************************************************//
//* FRIDA: flexible rapid interactive data analysis                        *//
//* olf.cpp: online files                                                  *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://www.mesen-und-deuten.de/frida                                   *//
//**************************************************************************//

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <algorithm>

#include <sys/types.h> // for fifo
#include <sys/stat.h>

#include <ask_simple_value.h>

using namespace std;

#include "mystd.h"
#include "olf.h"
#include "var.h"
#include "expr.h"


//***************************************************************************//
//* class COld                                                              *//
//***************************************************************************//

COld::COld( class COlc const* c )
{
    name = c->name;
    lDoc = c->lDoc;
    xco = c->xco;
    yco = c->yco;
    ZCo = c->ZCo;
    RPar = c->RPar;
}

uint COld::nPts( uint j ) const {
    if ( !nJ() || j>=nJ() )
        throw string( "BUG: nPts called with invalid j="+strg(j) );
    return VS(j)->size();
}


//! Return number of points if it is the same for all spectra. Else return 0.

uint COld::nPts() const {
    if ( !nJ() )
        throw string( "BUG: nPts() called while nSpec=0" );
    uint np = VS(0)->size();
    for ( uint j=0; j<nJ(); ++j )
        if (VS(j)->size()!=np) return 0;
    return np;
}


//***************************************************************************//
//* trivially duplicated COld/COlc functions                                *//
//***************************************************************************//


//! New shared pointer to copied spec/curve.

PZentry COld::new_zentry( uint j ) const
{
    return PZentry( new CSpec( *(VS(j)) ) );
}

PZentry COlc::new_zentry( uint j ) const
{
    return PZentry( new CCurve( *(VC(j)) ) );
}


//! New shared pointer to copied online file (but without copying main vector).

POlo COld::new_olo() const
{
    POlo ret( new COld( *this ) );
    ret->V.clear();
    ret->as_on_disk = false;
    return ret;
}

POlo COlc::new_olo() const
{
    POlo ret( new COlc( *this ) );
    ret->V.clear();
    ret->as_on_disk = false;
    return ret;
}

POld COld::new_old() const
{
    return P2D( new_olo() );
}

POlc COlc::new_olc() const
{
    return P2C( new_olo() );
}


//! Set main vector by copying from fin.

void COld::copy_mainvec( POlo fin )
{
    POld fd = boost::dynamic_pointer_cast<class COld>( fin );
    V.resize( fin->nJ() );
    for( uint j=0; j<fin->nJ(); ++j )
        V[j] = PZentry( new CSpec( *(fd->VS(j)) ) );
}

void COlc::copy_mainvec( POlo fin )
{
    POlc fc = boost::dynamic_pointer_cast<class COlc>( fin );
    V.resize( fin->nJ() );
    for( uint j=0; j<fin->nJ(); ++j )
        V[j] = PZentry( new CCurve( *(fc->VC(j)) ) );
}


//! Return pointer to selected main vector entry.

PSpec COld::VS( uint j ) const
{
    if( j>=V.size() )
        throw "VS call with j=" + strg(j) + " though V.size=" + strg(V.size());
    PSpec p = boost::dynamic_pointer_cast<class CSpec>( V[j] );
    if ( !p )
        throw string( "invalid VS call" );
    return p;
}

PCurve COlc::VC( uint j ) const
{
    if( j>=V.size() )
        throw "VC call with j=" + strg(j) + " though V.size=" + strg(V.size());
    PCurve p = boost::dynamic_pointer_cast<class CCurve>( V[j] );
    if ( !p )
        throw string( "invalid VC call" );
        return p;
}


//! Return one coordinate.

CCoord COld::coord( CVariable* var ) const
{
    if        ( var->typ==CVariable::_X ) {
        return xco;
    } else if ( var->typ==CVariable::_Y ) {
        return yco;
    } else if ( var->typ==CVariable::_Z ) {
        if( var->num >= nZ() )
            throw "coordinate " + var->var_info() + " not defined";
        return ZCo[var->num];
    } else if ( var->typ==CVariable::_R ) {
        if( var->num >= RPar.size() )
            throw "coordinate " + var->var_info() + " not defined";
        return RPar[var->num].Co;
    } else {
        throw "variable " + var->var_info() + " not defined for curve file";
    }
}

CCoord COlc::coord( CVariable* var ) const
{
    if        ( var->typ==CVariable::_X ) {
        return xco;
    } else if ( var->typ==CVariable::_Y ) {
        return yco;
    } else if ( var->typ==CVariable::_Z ) {
        if( var->num >= nZ() )
            throw "coordinate " + var->var_info() + " not defined";
        return ZCo[var->num];
    } else if ( var->typ==CVariable::_CP ) {
        if( var->num >= nPar() )
            throw "coordinate " + var->var_info() + " not defined";
        return PCo[var->num];
    } else if ( var->typ==CVariable::_R ) {
        if( var->num >= RPar.size() )
            throw "coordinate " + var->var_info() + " not defined";
        return RPar[var->num].Co;
    } else {
        throw "variable " + var->var_info() + " not defined for curve file";
    }
}


//! Change one coordinate.

void COld::setCoord( CVariable* var, CCoord& co )
{
    if        ( var->typ==CVariable::_X ) {
        xco = co;
    } else if ( var->typ==CVariable::_Y ) {
        yco = co;
    } else if ( var->typ==CVariable::_Z ) {
        if( var->num >= nZ() )
            throw "coordinate " + var->var_info() + " not defined";
        ZCo[var->num] = co;
    } else if ( var->typ==CVariable::_R ) {
        if( var->num >= RPar.size() )
            throw "coordinate " + var->var_info() + " not defined";
        RPar[var->num] = co;
    } else {
        throw "variable " + var->var_info() + " not defined for curve file";
    }
}

void COlc::setCoord( CVariable* var, CCoord& co )
{
    if        ( var->typ==CVariable::_X ) {
        xco = co;
    } else if ( var->typ==CVariable::_Y ) {
        yco = co;
    } else if ( var->typ==CVariable::_Z ) {
        if( var->num >= nZ() )
            throw "coordinate " + var->var_info() + " not defined";
        ZCo[var->num] = co;
    } else if ( var->typ==CVariable::_CP ) {
        if( var->num >= nPar() )
            throw "coordinate " + var->var_info() + " not defined";
        PCo[var->num] = co;
    } else if ( var->typ==CVariable::_R ) {
        if( var->num >= RPar.size() )
            throw "coordinate " + var->var_info() + " not defined";
        RPar[var->num] = co;
    } else {
        throw "variable " + var->var_info() + " not defined for curve file";
    }
}


//***************************************************************************//
//* Curve evaluation                                                        *//
//***************************************************************************//


//! Evaluate a curve, vectorial version. Used in fitting and in plotting.

void COlc::curve_val_vec( vector<double>* ret, const vector<double>& vt,
                          uint k, uint j ) const
  // input: the function arguments vt[], and the context k, j
  // output: the curve evaluation result *ret[]
{
    ret->resize( vt.size() );
    CResult val;
    if ( evaMode==_EXPR ) {
        cout << "DEB expr\n";
        CContext ctx( k, j );
        if        ( !T->has_dummy() ) {  // curve does not depend on t
            T->tree_val( val, ctx );
            if ( val.vectorial )
                throw "BUG: tree_curve_val did not get scalar result";
            (*ret)[0] = val.r;
        } else {
            ctx.request_VT( &vt );
            T->tree_val( val, ctx );
            if ( !val.vectorial )
                throw "BUG: tree_curve_val did not get vectorial result";
            *ret = val.v;
        }
    } else if ( evaMode==_SCR_INTP ) {
        string args;
        for( uint ip=0; ip<nPar(); ++ip )
            args += " " + strg( VC(j)->P[ip] );
        //cout << "DEB mkfifo\n";
        //mkfifo( "/tmp/f2curve", 0666 );
        string cmd = "'" + expr + args + "' > /ram/f2curve";
        cout << "DEB system [" + cmd + "]\n";
        system( cmd.c_str() );
        cout << "DEB read\n";
        FILE *F;
        if( !(F = fopen("/ram/f2curve", "r")) )
            throw "BUG: cannot read results of system call";
        CSpec S;
        double x, y;
        cout << "DEB reading ...\n";
        while( fscanf( F, "%lg %lg\n", &x, &y )==2 ){
            S.push_xy( x, y );
        }
        cout << "DEB no lines: " << S.size() << "\n";
        // interpolation:
        S.intpol( vt, *ret );
    } else
        throw "BUG: unexpected evaluation mode";
}


//! Dito, scalar version, to obtain isolated points.

double COlc::curve_val_sca( double farg, uint k, uint j ) const
{
    vector<double> vt(1, farg), res(1);
    curve_val_vec( &res, vt, k, j );
    return res[0];
}
